/*
 * Arduino Clock with Ring, Strip, LCD 128X64, dfPlayer, IR Remote, Temperature, DCF77, 
 * (PART 1/2)
 * The device will be powered by a 5 volt battery.
 * http://www.forgani.com/electronics-projects/
*/
#include <RTClib.h>
#include <TimeLib.h>
RTC_DS1307 RTC; // establish clock object
uint8_t mins, secs, hrs;
const char* dayName[7] = {"Sunday", "Monday", "Tuesday", "Wednesday", "Thursday", "Friday", "Saturday"};
const char* monName[12] = {"January", "February", "March", "April", "May", "June", "July", "August", "September", "October", "November", "December"};

#include "Wire.h"
#include <OneWire.h>
#include <DallasTemperature.h>
OneWire oneWire(4); // ONE_WIRE_BUS - DS18B20_PIN 4
DallasTemperature sensors(&oneWire);

float angle = 0;

#include <U8glib.h>
U8GLIB_ST7920_128X64_1X u8g(10); // CS_PIN 10

#include <DCF77.h>
#include <Time.h>      
DCF77 DCF=DCF77(2,0); // DCF_PIN,DCF_INTERRUPT - DCF77PIN 2
unsigned long lastDCF=0;
time_t DCFtime;

void setup() {
  //Serial.begin(9600);
  DCF.Start();
  Wire.begin();
  if (!RTC.begin()) {
    //Serial.println("Couldn't find RTC");
    while(1);
  }
  pinMode(6, OUTPUT); // DCFBlinkPin
  digitalWrite(6,1);
  delay(1000);
  digitalWrite(6,0);
  sensors.begin();  // DS18B20 starten
  delay(10);
}

byte X2 = 0;
byte Y2 = 0;
byte X3 = 0;
byte Y3 = 0;
byte ScreenWith = 128;
byte ScreenWithC = 102;
byte ScreenHeight = 64;
byte ScreenHeightC = 37;

/* Main program */
void loop() {
  if (now()-lastDCF > 3600 || lastDCF==0) setDateDs1307(); 
  uint8_t DCFSignalState=digitalRead(2) ^ 1;
  digitalWrite(6, DCFSignalState); // DCFBlinkPin
  DateTime now = RTC.now();
  sensors.requestTemperatures();
  secs = now.second();
  mins = now.minute();
  hrs = now.hour();
  /*
    Serial.print(hrs);
    Serial.print(":");
    Serial.print(mins);
    Serial.print(":");
    Serial.println(secs);
    Serial.print("Temperatur: ");
    Serial.print(now.dayOfTheWeek());
    Serial.print(" ");
    Serial.println("C");
  */
  u8g.firstPage();
  do {
    u8g.setColorIndex(1);
    u8g.setFont(u8g_font_6x13);
    u8g.setPrintPos(2, 36);
    u8g.print(dayName[now.dayOfTheWeek()]);
    u8g.setFont(u8g_font_5x7);
    u8g.setPrintPos(2, 45);
    u8g.print(now.day());
    u8g.setPrintPos(15, 45);
    u8g.print(monName[(now.month() - 1)]);
    u8g.setPrintPos(53, 45);
    u8g.print(now.year());
    //show time
    u8g.setFont(u8g_font_fub17);
    u8g.drawLine(2, 22, 60, 22);
    u8g.drawLine(2, 23, 60, 23);
    byte horPos = 0;
    if (hrs < 10) {
      u8g.drawStr(1, 20, "0");
      horPos = 15;
    } else horPos = 1;
    u8g.setPrintPos(horPos, 20);
    u8g.print(hrs);
    u8g.drawStr(27, 17, ":");
    byte minPos = 0;
    if (mins < 10) {
      u8g.drawStr(35, 20, "0");
      minPos = 48; // 51
    } else minPos = 35;
    u8g.setPrintPos(minPos, 20);
    u8g.print(mins);
    u8g.setFont(u8g_font_9x15B);
    u8g.drawStr(63, 13, ":");
    byte secPos = 0;
    if (secs < 10) {
      u8g.drawStr(72, 15, "0");
      secPos = 82;
    } else secPos = 72;
    u8g.setPrintPos(secPos, 15);
    u8g.print(secs);

    u8g.setFont(u8g_font_9x15B);
    u8g.setPrintPos(2, 60);
    u8g.print(sensors.getTempCByIndex(0), 1);
    u8g.drawCircle(43, 52, 2);
    u8g.drawStr(48, 60, "C");
    u8g.drawDisc(ScreenWithC, ScreenHeightC, 23);
    u8g.setColorIndex(0);
    u8g.drawDisc(ScreenWithC, ScreenHeightC, 1.5);
    u8g.setFont(u8g_font_5x7);
    u8g.setFontPosTop();
    u8g.drawStr(ScreenWithC - 1, ScreenHeightC - 23 - 3, "."); //12
    u8g.drawStr(ScreenWithC + 23 - 4, ScreenHeightC - 6, "."); //3
    u8g.drawStr(ScreenWithC - 1, ScreenHeightC + 23 - 8, "."); //6
    u8g.drawStr(ScreenWithC - 23 + 1, ScreenHeightC - 6, "."); //9
    u8g.drawCircle(ScreenWithC, ScreenHeightC, 22);
    u8g.drawStr(ScreenWithC - 12, ScreenHeightC + 15 - 9, "4gani");

    draw_pointer(hrs - 1, 12, 11);
    draw_pointer(mins - 5, 60, 15);
    draw_pointer(secs - 5, 60, 18);
  } while (u8g.nextPage());
  delay(600);
}

void draw_pointer(uint8_t valor, uint8_t rotation, uint8_t radius) {
  angle = valor * 2.0 * 3.1415 / rotation - 1, 5707;
  X2 = ScreenWithC + radius * cos(angle);
  Y2 = ScreenHeightC + radius * sin(angle);
  u8g.drawLine(ScreenWithC, ScreenHeightC, X2, Y2);
}

void setDateDs1307(){
  DCFtime=DCF.getTime();
  //Serial.print("Check if new DCF77 time is available");
  if (DCFtime!=0) {
    //Serial.print("set DCF77 time!!");
    setTime(DCFtime);
    lastDCF=now();
    Wire.beginTransmission(0x68);  // DS1307_ADDRESS 0x68
    Wire.write(0x00); //stop 
    Wire.write(decToBcd(second(DCFtime)));
    Wire.write(decToBcd(minute(DCFtime)));
    Wire.write(decToBcd(hour(DCFtime)));
    Wire.write(decToBcd(weekday(DCFtime))); // SUN=7,MON=1...
    Wire.write(decToBcd(day(DCFtime)));
    Wire.write(decToBcd(month(DCFtime)));
    Wire.write(decToBcd(year(DCFtime)));
    Wire.endTransmission();
  }
}

byte decToBcd(byte val){
  return ( (val/10*16)+(val%10) );
}
byte bcdToDec(byte val){
  return ( (val/16*10)+(val%16) );
}
